# This script dumps the books stored in the Delicious Library database file
# as BibTex entries, ready to use with LaTeX.
# Adrian Kosmaczewski, March 3rd, 2007

require 'rexml/document'
require 'iconv'

include REXML

# Open the Delicious Library database file
file = File.new(ENV['HOME'] + '/Library/Application Support/Delicious Library/Library Media Data.xml')
xml = Document.new(file)
result = String.new

# Iterate over all the "book" elements (it might take a couple of seconds...)
xml.elements.each("//items/book") do |book|
  
  # Gather some information about each book
  authors = book.attributes["author"].split(/\n/) if book.attributes["author"]
  author = authors.join(" and ") if authors
  publisher = book.attributes["publisher"].strip if book.attributes["publisher"]
  asin = book.attributes["asin"]
  isbn = asin[0,1] + "-" + asin[1, 3] + "-" + asin[4, 5] + "-" + asin[9, 1] if asin
  title = book.attributes["title"].strip if book.attributes["title"]
  year = book.attributes["published"].split("-")[2] if book.attributes["published"]
  
  # Create a key for each entry, by default it's the ISBN number
  # Be careful! The keys might not be unique!
  key = asin
  if authors
    names = authors[0].split(" ")
    name = String.new
    names.each do |n|
      name = n if n.length > 3
    end
    key = name
    key << ";"
    key << year
  end
  
  # Output the BibTex entries with a space between each one
  result << "@book{#{key}, 
    Author = {#{author}},
    Publisher = {#{publisher}},
    Series = {ISBN #{isbn}},
    Title = {#{title}},
    Year = {#{year}}}\n\n"

end

# Change the output to plain ASCII
result = Iconv.conv("ASCII//TRANSLIT", "UTF-8", result)
puts result
